# Copyright 2022 The Chromium Authors
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.
"""Validate mojo attributes are allowed in Chrome before generation."""

import mojom.generate.check as check
import mojom.generate.module as module

_COMMON_ATTRIBUTES = {
    'EnableIf',
    'EnableIfNot',
}

# For struct, union & parameter lists.
_COMMON_FIELD_ATTRIBUTES = _COMMON_ATTRIBUTES | {
    'MinVersion',
    'RenamedFrom',
}

# Note: `Default`` goes on the default _value_, not on the enum.
# Note: [Stable] without [Extensible] is not allowed.
_ENUM_ATTRIBUTES = _COMMON_ATTRIBUTES | {
    'Extensible',
    'Native',
    'Stable',
    'RenamedFrom',
    'Uuid',
}

# TODO(crbug.com/1234883) MinVersion is not needed for EnumVal.
_ENUMVAL_ATTRIBUTES = _COMMON_ATTRIBUTES | {
    'Default',
    'MinVersion',
}

_INTERFACE_ATTRIBUTES = _COMMON_ATTRIBUTES | {
    'RenamedFrom',
    'RequireContext',
    'RuntimeFeature',
    'ServiceSandbox',
    'Stable',
    'Uuid',
}

_METHOD_ATTRIBUTES = _COMMON_ATTRIBUTES | {
    'AllowedContext',
    'MinVersion',
    'NoInterrupt',
    'RuntimeFeature',
    'SupportsUrgent',
    'Sync',
    'UnlimitedSize',
}

_MODULE_ATTRIBUTES = _COMMON_ATTRIBUTES | {
    'JavaConstantsClassName',
    'JavaPackage',
}

_PARAMETER_ATTRIBUTES = _COMMON_FIELD_ATTRIBUTES

_STRUCT_ATTRIBUTES = _COMMON_ATTRIBUTES | {
    'CustomSerializer',
    'JavaClassName',
    'Native',
    'Stable',
    'RenamedFrom',
    'Uuid',
}

_STRUCT_FIELD_ATTRIBUTES = _COMMON_FIELD_ATTRIBUTES

_UNION_ATTRIBUTES = _COMMON_ATTRIBUTES | {
    'Extensible',
    'Stable',
    'RenamedFrom',
    'Uuid',
}

_UNION_FIELD_ATTRIBUTES = _COMMON_FIELD_ATTRIBUTES | {
    'Default',
}

# TODO(https://crbug.com/1193875) empty this set and remove the allowlist.
_STABLE_ONLY_ALLOWLISTED_ENUMS = {
    'crosapi.mojom.OptionalBool',
    'crosapi.mojom.TriState',
}


class Check(check.Check):
  def __init__(self, *args, **kwargs):
    super(Check, self).__init__(*args, **kwargs)

  def _Respell(self, allowed, attribute):
    for a in allowed:
      if a.lower() == attribute.lower():
        return f" - Did you mean: {a}?"
    return ""

  def _CheckAttributes(self, context, allowed, attributes):
    if not attributes:
      return
    for attribute in attributes:
      if not attribute in allowed:
        # Is there a close misspelling?
        hint = self._Respell(allowed, attribute)
        raise check.CheckException(
            self.module,
            f"attribute {attribute} not allowed on {context}{hint}")

  def _CheckEnumAttributes(self, enum):
    if enum.attributes:
      self._CheckAttributes("enum", _ENUM_ATTRIBUTES, enum.attributes)
      if 'Stable' in enum.attributes and not 'Extensible' in enum.attributes:
        full_name = f"{self.module.mojom_namespace}.{enum.mojom_name}"
        if full_name not in _STABLE_ONLY_ALLOWLISTED_ENUMS:
          raise check.CheckException(
              self.module,
              f"[Extensible] required on [Stable] enum {full_name}")
    for enumval in enum.fields:
      self._CheckAttributes("enum value", _ENUMVAL_ATTRIBUTES,
                            enumval.attributes)

  def _CheckInterfaceAttributes(self, interface):
    self._CheckAttributes("interface", _INTERFACE_ATTRIBUTES,
                          interface.attributes)
    for method in interface.methods:
      self._CheckAttributes("method", _METHOD_ATTRIBUTES, method.attributes)
      for param in method.parameters:
        self._CheckAttributes("parameter", _PARAMETER_ATTRIBUTES,
                              param.attributes)
      if method.response_parameters:
        for param in method.response_parameters:
          self._CheckAttributes("parameter", _PARAMETER_ATTRIBUTES,
                                param.attributes)
    for enum in interface.enums:
      self._CheckEnumAttributes(enum)

  def _CheckModuleAttributes(self):
    self._CheckAttributes("module", _MODULE_ATTRIBUTES, self.module.attributes)

  def _CheckStructAttributes(self, struct):
    self._CheckAttributes("struct", _STRUCT_ATTRIBUTES, struct.attributes)
    for field in struct.fields:
      self._CheckAttributes("struct field", _STRUCT_FIELD_ATTRIBUTES,
                            field.attributes)
    for enum in struct.enums:
      self._CheckEnumAttributes(enum)

  def _CheckUnionAttributes(self, union):
    self._CheckAttributes("union", _UNION_ATTRIBUTES, union.attributes)
    for field in union.fields:
      self._CheckAttributes("union field", _UNION_FIELD_ATTRIBUTES,
                            field.attributes)

  def CheckModule(self):
    """Note that duplicate attributes are forbidden at the parse phase.
    We also do not need to look at the types of any parameters, as they will be
    checked where they are defined. Consts do not have attributes so can be
    skipped."""
    self._CheckModuleAttributes()
    for interface in self.module.interfaces:
      self._CheckInterfaceAttributes(interface)
    for enum in self.module.enums:
      self._CheckEnumAttributes(enum)
    for struct in self.module.structs:
      self._CheckStructAttributes(struct)
    for union in self.module.unions:
      self._CheckUnionAttributes(union)



#!/usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-or-later
# Copyright (C) 2021, Google Inc.
#
# Author: Paul Elder <paul.elder@ideasonboard.com>
#
# extract-docs.py - Extract doxygen documentation from mojom files

import argparse
import re
import sys

regex_block_start = re.compile('^\/\*\*$')
regex_block_end = re.compile('^ \*\/$')
regex_spdx = re.compile('^\/\* SPDX-License-Identifier: .* \*\/$')


def main(argv):

    # Parse command line arguments
    parser = argparse.ArgumentParser()
    parser.add_argument('-o', dest='output', metavar='file',
                        type=argparse.FileType('w', encoding='utf-8'),
                        default=sys.stdout,
                        help='Output file name (default: standard output)')
    parser.add_argument('input', type=str,
                        help='Input file name.')
    args = parser.parse_args(argv[1:])

    lines = open(args.input, 'r').readlines()
    pipeline = args.input.split('/')[-1].replace('.mojom', '')

    if not regex_spdx.match(lines[0]):
        raise Exception(f'Missing SPDX license header in {args.input}')

    data = lines[0]
    data += f'''\
/*
 * Copyright (C) 2021, Google Inc.
 *
 * {pipeline}_ipa_interface.cpp - Docs file for generated {pipeline}.mojom
 *
 * This file is auto-generated. Do not edit.
 */

namespace libcamera {{

'''

    in_block = False
    comment = ''
    for lineno, line in enumerate(lines, start=1):
        if regex_block_start.match(line):
            if in_block:
                raise SyntaxError('Expected end of comment',
                                  (args.input, lineno, 1, line))
            in_block = True
            comment = line
            continue

        if regex_block_end.match(line):
            if in_block:
                comment += line
                data += comment + '\n'
            in_block = False
            continue

        if in_block:
            comment += line

    data += '} /* namespace libcamera */\n'

    args.output.write(data)

    return 0


if __name__ == '__main__':
    sys.exit(main(sys.argv))



# SPDX-License-Identifier: BSD-2-Clause
#
# Copyright (C) 2019, Raspberry Pi Ltd
# Copyright (C) 2022, Paul Elder <paul.elder@ideasonboard.com>

from ..module import Module

import libtuning as lt
import libtuning.utils as utils

import numpy as np


class LSC(Module):
    type = 'lsc'
    hr_name = 'LSC (Base)'
    out_name = 'GenericLSC'

    def __init__(self, *,
                 debug: list,
                 sector_shape: tuple,
                 sector_x_gradient: lt.Gradient,
                 sector_y_gradient: lt.Gradient,
                 sector_average_function: lt.Average,
                 smoothing_function: lt.Smoothing):
        super().__init__()

        self.debug = debug

        self.sector_shape = sector_shape
        self.sector_x_gradient = sector_x_gradient
        self.sector_y_gradient = sector_y_gradient
        self.sector_average_function = sector_average_function

        self.smoothing_function = smoothing_function

    def _enumerate_lsc_images(self, images):
        for image in images:
            if image.lsc_only:
                yield image

    def _get_grid(self, channel, img_w, img_h):
        # List of number of pixels in each sector
        sectors_x = self.sector_x_gradient.distribute(img_w / 2, self.sector_shape[0])
        sectors_y = self.sector_y_gradient.distribute(img_h / 2, self.sector_shape[1])

        grid = []

        r = 0
        for y in sectors_y:
            c = 0
            for x in sectors_x:
                grid.append(self.sector_average_function.average(channel[r:r + y, c:c + x]))
                c += x
            r += y

        return np.array(grid)

    def _lsc_single_channel(self, channel: np.array,
                            image: lt.Image, green_grid: np.array = None):
        grid = self._get_grid(channel, image.w, image.h)
        grid -= image.blacklevel_16
        if green_grid is None:
            table = np.reshape(1 / grid, self.sector_shape[::-1])
        else:
            table = np.reshape(green_grid / grid, self.sector_shape[::-1])
        table = self.smoothing_function.smoothing(table)

        if green_grid is None:
            table = table / np.min(table)

        return table, grid



#!/usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-or-later
# Copyright (C) 2019, Google Inc.
#
# Author: Laurent Pinchart <laurent.pinchart@ideasonboard.com>
#
# gen-controls.py - Generate control definitions from YAML

import argparse
from functools import reduce
import operator
import string
import sys
import yaml
import os


class ControlEnum(object):
    def __init__(self, data):
        self.__data = data

    @property
    def description(self):
        """The enum description"""
        return self.__data.get('description')

    @property
    def name(self):
        """The enum name"""
        return self.__data.get('name')

    @property
    def value(self):
        """The enum value"""
        return self.__data.get('value')


class Control(object):
    def __init__(self, name, data, vendor):
        self.__name = name
        self.__data = data
        self.__enum_values = None
        self.__size = None
        self.__vendor = vendor

        enum_values = data.get('enum')
        if enum_values is not None:
            self.__enum_values = [ControlEnum(enum) for enum in enum_values]

        size = self.__data.get('size')
        if size is not None:
            if len(size) == 0:
                raise RuntimeError(f'Control `{self.__name}` size must have at least one dimension')

            # Compute the total number of elements in the array. If any of the
            # array dimension is a string, the array is variable-sized.
            num_elems = 1
            for dim in size:
                if type(dim) is str:
                    num_elems = 0
                    break

                dim = int(dim)
                if dim <= 0:
                    raise RuntimeError(f'Control `{self.__name}` size must have positive values only')

                num_elems *= dim

            self.__size = num_elems

    @property
    def description(self):
        """The control description"""
        return self.__data.get('description')

    @property
    def enum_values(self):
        """The enum values, if the control is an enumeration"""
        if self.__enum_values is None:
            return
        for enum in self.__enum_values:
            yield enum

    @property
    def is_enum(self):
        """Is the control an enumeration"""
        return self.__enum_values is not None

    @property
    def vendor(self):
        """The vendor string, or None"""
        return self.__vendor

    @property
    def name(self):
        """The control name (CamelCase)"""
        return self.__name

    @property
    def type(self):
        typ = self.__data.get('type')
        size = self.__data.get('size')

        if typ == 'string':
            return 'std::string'

        if self.__size is None:
            return typ

        if self.__size:
            return f"Span<const {typ}, {self.__size}>"
        else:
            return f"Span<const {typ}>"


def snake_case(s):
    return ''.join([c.isupper() and ('_' + c) or c for c in s]).strip('_')


def format_description(description):
    description = description.strip('\n').split('\n')
    description[0] = '\\brief ' + description[0]
    return '\n'.join([(line and ' * ' or ' *') + line for line in description])


def generate_cpp(controls):
    enum_doc_start_template = string.Template('''/**
 * \\enum ${name}Enum
 * \\brief Supported ${name} values''')
    enum_doc_value_template = string.Template(''' * \\var ${value}
${description}''')
    doc_template = string.Template('''/**
 * \\var ${name}
${description}
 */''')
    def_template = string.Template('extern const Control<${type}> ${name}(${id_name}, "${name}");')
    enum_values_doc = string.Template('''/**
 * \\var ${name}Values
 * \\brief List of all $name supported values
 */''')
    enum_values_start = string.Template('''extern const std::array<const ControlValue, ${size}> ${name}Values = {''')
    enum_values_values = string.Template('''\tstatic_cast<int32_t>(${name}),''')

    ctrls_doc = {}
    ctrls_def = {}
    ctrls_map = []

    for ctrl in controls:
        id_name = snake_case(ctrl.name).upper()

        vendor = ctrl.vendor
        if vendor not in ctrls_doc:
            ctrls_doc[vendor] = []
            ctrls_def[vendor] = []

        info = {
            'name': ctrl.name,
            'type': ctrl.type,
            'description': format_description(ctrl.description),
            'id_name': id_name,
        }

        target_doc = ctrls_doc[vendor]
        target_def = ctrls_def[vendor]

        if ctrl.is_enum:
            enum_doc = []
            enum_doc.append(enum_doc_start_template.substitute(info))

            num_entries = 0
            for enum in ctrl.enum_values:
                value_info = {
                    'name': ctrl.name,
                    'value': enum.name,
                    'description': format_description(enum.description),
                }
                enum_doc.append(enum_doc_value_template.substitute(value_info))
                num_entries += 1

            enum_doc = '\n *\n'.join(enum_doc)
            enum_doc += '\n */'
            target_doc.append(enum_doc)

            values_info = {
                'name': info['name'],
                'size': num_entries,
            }
            target_doc.append(enum_values_doc.substitute(values_info))
            target_def.append(enum_values_start.substitute(values_info))
            for enum in ctrl.enum_values:
                value_info = {
                    'name': enum.name
                }
                target_def.append(enum_values_values.substitute(value_info))
            target_def.append("};")

        target_doc.append(doc_template.substitute(info))
        target_def.append(def_template.substitute(info))

        vendor_ns = vendor + '::' if vendor != "libcamera" else ''
        ctrls_map.append('\t{ ' + vendor_ns + id_name + ', &' + vendor_ns + ctrl.name + ' },')

    vendor_ctrl_doc_sub = []
    vendor_ctrl_template = string.Template('''
/**
 * \\brief Namespace for ${vendor} controls
 */
namespace ${vendor} {

${vendor_controls_str}

} /* namespace ${vendor} */''')

    for vendor in [v for v in ctrls_doc.keys() if v not in ['libcamera']]:
        vendor_ctrl_doc_sub.append(vendor_ctrl_template.substitute({'vendor': vendor, 'vendor_controls_str': '\n\n'.join(ctrls_doc[vendor])}))

    vendor_ctrl_def_sub = []
    for vendor in [v for v in ctrls_def.keys() if v not in ['libcamera']]:
        vendor_ctrl_def_sub.append(vendor_ctrl_template.substitute({'vendor': vendor, 'vendor_controls_str': '\n'.join(ctrls_def[vendor])}))

    return {
        'controls_doc': '\n\n'.join(ctrls_doc['libcamera']),
        'controls_def': '\n'.join(ctrls_def['libcamera']),
        'controls_map': '\n'.join(ctrls_map),
        'vendor_controls_doc': '\n'.join(vendor_ctrl_doc_sub),
        'vendor_controls_def': '\n'.join(vendor_ctrl_def_sub),
    }


def generate_h(controls, mode, ranges):
    enum_template_start = string.Template('''enum ${name}Enum {''')
    enum_value_template = string.Template('''\t${name} = ${value},''')
    enum_values_template = string.Template('''extern const std::array<const ControlValue, ${size}> ${name}Values;''')
    template = string.Template('''extern const Control<${type}> ${name};''')

    ctrls = {}
    ids = {}
    id_value = {}

    for ctrl in controls:
        id_name = snake_case(ctrl.name).upper()

        vendor = ctrl.vendor
        if vendor not in ctrls:
            if vendor not in ranges.keys():
                raise RuntimeError(f'Control id range is not defined for vendor {vendor}')
            id_value[vendor] = ranges[vendor] + 1
            ids[vendor] = []
            ctrls[vendor] = []

        target_ids = ids[vendor]
        target_ids.append('\t' + id_name + ' = ' + str(id_value[vendor]) + ',')

        info = {
            'name': ctrl.name,
            'type': ctrl.type,
        }

        target_ctrls = ctrls[vendor]

        if ctrl.is_enum:
            target_ctrls.append(enum_template_start.substitute(info))

            num_entries = 0
            for enum in ctrl.enum_values:
                value_info = {
                    'name': enum.name,
                    'value': enum.value,
                }
                target_ctrls.append(enum_value_template.substitute(value_info))
                num_entries += 1
            target_ctrls.append("};")

            values_info = {
                'name': info['name'],
                'size': num_entries,
            }
            target_ctrls.append(enum_values_template.substitute(values_info))

        target_ctrls.append(template.substitute(info))
        id_value[vendor] += 1

    vendor_template = string.Template('''
namespace ${vendor} {

#define LIBCAMERA_HAS_${vendor_def}_VENDOR_${mode}

enum {
${vendor_enums}
};

${vendor_controls}

} /* namespace ${vendor} */
''')

    vendor_sub = []
    for vendor in [v for v in ctrls.keys() if v != 'libcamera']:
        vendor_sub.append(vendor_template.substitute({'mode': mode.upper(),
                                                      'vendor': vendor,
                                                      'vendor_def': vendor.upper(),
                                                      'vendor_enums': '\n'.join(ids[vendor]),
                                                      'vendor_controls': '\n'.join(ctrls[vendor])}))

    return {
        'ids': '\n'.join(ids['libcamera']),
        'controls': '\n'.join(ctrls['libcamera']),
        'vendor_controls': '\n'.join(vendor_sub)
    }


def fill_template(template, data):

    template = open(template, 'rb').read()
    template = template.decode('utf-8')
    template = string.Template(template)
    return template.substitute(data)


def main(argv):

    # Parse command line arguments
    parser = argparse.ArgumentParser()
    parser.add_argument('--mode', '-m', type=str, required=True, choices=['controls', 'properties'],
                        help='Mode of operation')
    parser.add_argument('--output', '-o', metavar='file', type=str,
                        help='Output file name. Defaults to standard output if not specified.')
    parser.add_argument('--ranges', '-r', type=str, required=True,
                        help='Control id range reservation file.')
    parser.add_argument('--template', '-t', dest='template', type=str, required=True,
                        help='Template file name.')
    parser.add_argument('input', type=str, nargs='+',
                        help='Input file name.')

    args = parser.parse_args(argv[1:])

    ranges = {}
    with open(args.ranges, 'rb') as f:
        data = open(args.ranges, 'rb').read()
        ranges = yaml.safe_load(data)['ranges']

    controls = []
    for input in args.input:
        with open(input, 'rb') as f:
            data = f.read()
            vendor = yaml.safe_load(data)['vendor']
            ctrls = yaml.safe_load(data)['controls']
            controls = controls + [Control(*ctrl.popitem(), vendor) for ctrl in ctrls]

    if args.template.endswith('.cpp.in'):
        data = generate_cpp(controls)
    elif args.template.endswith('.h.in'):
        data = generate_h(controls, args.mode, ranges)
    else:
        raise RuntimeError('Unknown template type')

    data = fill_template(args.template, data)

    if args.output:
        output = open(args.output, 'wb')
        output.write(data.encode('utf-8'))
        output.close()
    else:
        sys.stdout.write(data)

    return 0


if __name__ == '__main__':
    sys.exit(main(sys.argv))



# Copyright 2013 The Chromium Authors
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

# Based on third_party/WebKit/Source/build/scripts/template_expander.py.

import os.path
import sys

from mojom import fileutil

fileutil.AddLocalRepoThirdPartyDirToModulePath()
import jinja2


def ApplyTemplate(mojo_generator, path_to_template, params, **kwargs):
  loader = jinja2.ModuleLoader(
      os.path.join(mojo_generator.bytecode_path,
                   "%s.zip" % mojo_generator.GetTemplatePrefix()))
  final_kwargs = dict(mojo_generator.GetJinjaParameters())
  final_kwargs.update(kwargs)

  jinja_env = jinja2.Environment(
      loader=loader, keep_trailing_newline=True, **final_kwargs)
  jinja_env.globals.update(mojo_generator.GetGlobals())
  jinja_env.filters.update(mojo_generator.GetFilters())
  template = jinja_env.get_template(path_to_template)
  return template.render(params)


def UseJinja(path_to_template, **kwargs):
  def RealDecorator(generator):
    def GeneratorInternal(*args, **kwargs2):
      parameters = generator(*args, **kwargs2)
      return ApplyTemplate(args[0], path_to_template, parameters, **kwargs)

    GeneratorInternal.__name__ = generator.__name__
    return GeneratorInternal

  return RealDecorator


def ApplyImportedTemplate(mojo_generator, path_to_template, filename, params,
                          **kwargs):
  loader = jinja2.FileSystemLoader(searchpath=path_to_template)
  final_kwargs = dict(mojo_generator.GetJinjaParameters())
  final_kwargs.update(kwargs)

  jinja_env = jinja2.Environment(
      loader=loader, keep_trailing_newline=True, **final_kwargs)
  jinja_env.globals.update(mojo_generator.GetGlobals())
  jinja_env.filters.update(mojo_generator.GetFilters())
  template = jinja_env.get_template(filename)
  return template.render(params)


def UseJinjaForImportedTemplate(func):
  def wrapper(*args, **kwargs):
    parameters = func(*args, **kwargs)
    path_to_template = args[1]
    filename = args[2]
    return ApplyImportedTemplate(args[0], path_to_template, filename,
                                 parameters)

  wrapper.__name__ = func.__name__
  return wrapper


def PrecompileTemplates(generator_modules, output_dir):
  for module in generator_modules.values():
    generator = module.Generator(None)
    jinja_env = jinja2.Environment(
        loader=jinja2.FileSystemLoader([
            os.path.join(
                os.path.dirname(module.__file__), generator.GetTemplatePrefix())
        ]))
    jinja_env.filters.update(generator.GetFilters())
    jinja_env.compile_templates(os.path.join(
        output_dir, "%s.zip" % generator.GetTemplatePrefix()),
                                extensions=["tmpl"],
                                zip="stored",
                                ignore_errors=False)



# Copyright 2018 The Chromium Authors
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.
"""Helpers for processing conditionally enabled features in a mojom."""

from mojom.error import Error
from mojom.parse import ast


class EnableIfError(Error):
  """ Class for errors from ."""

  def __init__(self, filename, message, lineno=None):
    Error.__init__(self, filename, message, lineno=lineno, addenda=None)


def _IsEnabled(definition, enabled_features):
  """Returns true if a definition is enabled.

  A definition is enabled if it has no EnableIf/EnableIfNot attribute.
  It is retained if it has an EnableIf attribute and the attribute is in
  enabled_features. It is retained if it has an EnableIfNot attribute and the
  attribute is not in enabled features.
  """
  if not hasattr(definition, "attribute_list"):
    return True
  if not definition.attribute_list:
    return True

  already_defined = False
  for a in definition.attribute_list:
    if a.key == 'EnableIf' or a.key == 'EnableIfNot':
      if already_defined:
        raise EnableIfError(
            definition.filename,
            "EnableIf/EnableIfNot attribute may only be set once per field.",
            definition.lineno)
      already_defined = True

  for attribute in definition.attribute_list:
    if attribute.key == 'EnableIf' and attribute.value not in enabled_features:
      return False
    if attribute.key == 'EnableIfNot' and attribute.value in enabled_features:
      return False
  return True


def _FilterDisabledFromNodeList(node_list, enabled_features):
  if not node_list:
    return
  assert isinstance(node_list, ast.NodeListBase)
  node_list.items = [
      item for item in node_list.items if _IsEnabled(item, enabled_features)
  ]
  for item in node_list.items:
    _FilterDefinition(item, enabled_features)


def _FilterDefinition(definition, enabled_features):
  """Filters definitions with a body."""
  if isinstance(definition, ast.Enum):
    _FilterDisabledFromNodeList(definition.enum_value_list, enabled_features)
  elif isinstance(definition, ast.Method):
    _FilterDisabledFromNodeList(definition.parameter_list, enabled_features)
    _FilterDisabledFromNodeList(definition.response_parameter_list,
                                enabled_features)
  elif isinstance(definition,
                  (ast.Interface, ast.Struct, ast.Union, ast.Feature)):
    _FilterDisabledFromNodeList(definition.body, enabled_features)


def RemoveDisabledDefinitions(mojom, enabled_features):
  """Removes conditionally disabled definitions from a Mojom node."""
  mojom.import_list = ast.ImportList([
      imported_file for imported_file in mojom.import_list
      if _IsEnabled(imported_file, enabled_features)
  ])
  mojom.definition_list = [
      definition for definition in mojom.definition_list
      if _IsEnabled(definition, enabled_features)
  ]
  for definition in mojom.definition_list:
    _FilterDefinition(definition, enabled_features)
